import {dispatch, select, subscribe} from "@wordpress/data";

import {
    ATOMICREACH_API_ANALYZE_ARTICLES,
    ATOMICREACH_API_NAMESPACE,
    ATOMICREACH_API_OPTIMIZE_ARTICLES,
    ATOMICREACH_API_PARAGRAPH_ANALYZE_ARTICLES,
    ATOMICREACH_SIDEBAR_NAME,
    ATOMICREACH_STORE,
    ATOMICREACH_VALID_BLOCK_NAMES_ARRAY,
    COMPLIANCE_CLASS,
    MARK_TAG_NAME,
    OPTIMIZED_WORD_CLASS,
    SPELLING_GRAMMAR_CLASS,
} from "../constants";
import {Button, SelectControl} from '@wordpress/components';
import {
    cleanARHighlightingFromBlock,
    debugCountHighlightedWords,
    getHTMLFromBlock,
    highlightComplianceList,
    highlightParagraphLength,
    highlightSpellingGrammar,
    removeARMarkerTags,
    updateBlockContent
} from '../library/utils';
// import Tooltip from './Tooltip'
import {addToDictionary, getPopoverState, replaceActiveWord} from '../library/EditorLib';
import apiFetch from "@wordpress/api-fetch";
import {doRePositioningXYEditor, flagWordEditor} from '../library/popoverLib';
import GenericPopover from '../library/GenericPopover';
import SidebarTitleFeedback from "./SidebarTitleFeedback";
import SidebarBodyFeedback from "./SidebarBodyFeedback";
import {Component} from "@wordpress/element";
import TooltipContainer from './TooltipContainer'
//https://lodash.com/docs/
import {cloneDeep} from 'lodash'

class SidebarWrapper extends Component {

    constructor() {
        super(...arguments);

        this.state = {
            profiles: {},
            selectedProfile: null,
            selectedProfileDetails: {},
            optimizeButton: {
                disabled: true,
                isBusy: false,
                hasClicked: false,
            },
            revertButton: {
                disabled: false,
                visible: false,
            },
            tooltipMessage: 'Select a profile before optimizing',
            tooltipVisibility: true,
            taskListFeedback: {},
            optimizeFeedback: null,
            paragraphsFeedback: null,
            hasChangedContent: false,
            isTyping: false,
            title: '',
            content: '',
            popoverState: {},
            visibility: {},
            optimizing:false,
            analyzing:false,
        };

        this.optimziedMeasures = [
            'Arousal',
            'Readability'
        ];
        this.spellingIssueFeedback = {};
        this.clearingHiglights = false;
        this.atomicReachApiCallInProgress = false;
        this.componentIsMounted = true;
        this.showNotice = false;
        this.unsubscribe = () => null;

        this.optimizeFeedbackForRehighlighting = {};
        this.totalOptimizeWordsToHighlight = 0;


        this.titleMeasures = [
            'TitleCharacterCount',
            'TitleWordsCount',
            'TitleContains5W1H',
            'TitleContainsNumbers',
            'TitlePolarity',
            'TitlePronounPerson',
            'TitleQuestion',
            'TitleSuperlatives',
            'TitleTopicsCount',
            'TitleTopicLocation',

        ];

        this.bodyMeasures = [
            'PercentageParagraphsIdealWordCount',
            'WordsCount',
        ];

        this.engagementNames = {
            GA_pageviews: 'Page Views',
            GA_uniquepageviews: 'Unique Views',
            GA_goalcompletionsall: 'Total Conversions',
            GA_avgTimeOnPage: 'Avg. Time on Page',
            GA_timeonpage: 'Time on Page',
            HS_ampViews: 'AMP Views',
            HS_ctaclick: 'CTA Clicks',
            HS_ctaRate: 'CTA Rate',
            HS_ctaviews: 'CTA Views',
            HS_entrances: 'Page Entrances',
            HS_leads: 'Leads',
            HS_marketingQualifiedLeads: 'Marketing Qualified Leads',
            HS_opportunities: 'Marketing Opportunities',
            HS_others: 'Others',
            HS_submissions: 'Submissions',
            HS_subscribers: 'Subscribers',
            HS_pageviews: 'Views',
            HS_pageTime: 'Time on Page',
            HS_timePerPageview: 'Time Per Pagview ',
            HS_submissionsPerPageview: 'View to Submission Rate',
            HS_contactsPerPageview: 'View to Contact Rate',
        };


        this.cardSelectionCallback = this.cardSelectionCallback.bind(this);

    }


    componentDidMount() {
        // console.log("Sidebar::comp did mount!", this.props, this.state);

        const {
            taskListFeedback,
            optimizeFeedback,
            selectedProfile,
            optimizeFeedbackReHighlighting,
            optimizeButtonStatus,
            revertButtonStatus,
            tooltipMessage
        } = this.props;


        this.updateProfileState();
        this.updateTitleContentState();

        //https://developer.wordpress.org/block-editor/packages/packages-data/#subscribe
        this.unsubscribe = subscribe(() => this.subscribeListener());
        const hasChanges = this.checkForChanges();
        this.removeNotice('warningContent');


        //use all stored data
        if(Object.entries(taskListFeedback).length > 0 && !hasChanges) {


            this.optimizeFeedbackForRehighlighting = optimizeFeedbackReHighlighting;
            for(const ind in this.optimizeFeedbackForRehighlighting) {
                this.totalOptimizeWordsToHighlight += Object.keys(this.optimizeFeedbackForRehighlighting).length;
            }

            this.titleDocumentFeedback(taskListFeedback);
            this.updateSelectedProfile(selectedProfile.profile);
            this.reHighlightOptimizedWords(optimizeFeedback);

            this.setState({
                optimizeFeedback: optimizeFeedback,
                optimizeButton: {
                    ...this.state.optimizeButton,
                    hasClicked: true,
                    disabled: optimizeButtonStatus,

                },
                tooltipVisibility: (!!optimizeButtonStatus ),
                tooltipMessage: tooltipMessage,
                revertButton: {
                    disabled: revertButtonStatus,
                    visible: true,
                },
                taskListFeedback
            });

        } else {
            dispatch(ATOMICREACH_STORE).clearStore();
            this.callParagraphFeedback();
        }

        const editorContainer = document.querySelector('.editor-writing-flow');
        if(editorContainer !== null) {
            document.querySelector('.editor-writing-flow').addEventListener('click', (event) => {
                let mark = event.target;
                if (mark.classList.contains(SPELLING_GRAMMAR_CLASS)) {
                    mark.arSuggestions = this.spellingIssueFeedback[mark.getAttribute('data-spelling-issue-id')];
                } else if (mark.classList.contains(OPTIMIZED_WORD_CLASS)) {
                    mark.arSuggestions = this.state.optimizeFeedback[parseInt(mark.dataset.blockIndex)][parseInt(mark.dataset.optimizeId)];
                }
                if (mark.classList.contains(SPELLING_GRAMMAR_CLASS) || (mark.classList.contains(OPTIMIZED_WORD_CLASS) || (mark.classList.contains(COMPLIANCE_CLASS)))) {
                    const popoverState = getPopoverState(mark);
                    this.setState({popoverState});
                }
            });

            //@todo: added post listener because the subscribe function is triggered by to many things we needed a solution that can pinpoint the pasting issue
            //@todo: If becomes a wordpress api that accounts for pasting that should replace this

            document.querySelector('.editor-writing-flow').addEventListener('paste', () => {
                this.reEnableOptimizeButton()
            })

        }
    }


    componentWillUnmount() {

        const {
            optimizeButton,
            revertButton,
            tooltipMessage,
        } =this.state;

        dispatch(ATOMICREACH_STORE).setOptimizeFeedbackReHighlighting({
            optimizeFeedbackReHighlighting: this.optimizeFeedbackForRehighlighting
        });

        dispatch(ATOMICREACH_STORE).setOptimizeButtonStatus(optimizeButton.disabled);

        dispatch(ATOMICREACH_STORE).setRevertButtonStatus(revertButton.disabled);

        dispatch(ATOMICREACH_STORE).setTooltipMessage(tooltipMessage);

        this.unsubscribe();
        this.removeARHighlighting();
        this.removeNotice('publishReadyNotice');
        this.removeNotice('warningContent');

        if(this.domHasOptimizedWords()!== 0) {

            dispatch("core/notices")
                .createWarningNotice(`Warning: Underlining of optimized words will be lost if article is changed outside of the Atomic Reach tab`, {
                    id: 'warningContent'
                })
        }else{
            dispatch(ATOMICREACH_STORE).setOptimizeFeedback({});
            dispatch(ATOMICREACH_STORE).setOptimizeFeedbackReHighlighting({optimizeFeedbackReHighlighting:{}});
        }

        this.takeSnapShop();
        this.componentIsMounted = false;
    }


    componentDidUpdate(prevProps, prevState, snapshot) {

        const {profiles} = this.props;

        if(profiles.length !== prevProps.profiles.length) {
            this.updateProfileState();
        }

        const {
            hasChangedContent,
            paragraphsFeedback,
            selectedProfile,
            optimizeFeedback,
            optimizeButton,
            optimizing,
            analyzing,
        } = this.state;




        if(hasChangedContent && !prevState.hasChangedContent) {

            const intervalToCallApi = setInterval(() => {
                if(!select('core/editor').isTyping() && !this.getAtomicReachApiCallInProgress()) {

                    this.updateTitleContentState();
                    clearInterval(intervalToCallApi);

                    if(selectedProfile === null) {
                        // even if no prof is select, still call the api to get spelling/grammar
                        this.callParagraphFeedback();
                    } else {
                        this.callTitleDocumentApi();
                    }
                }
            }, 500);
        }



        if(JSON.stringify(paragraphsFeedback) !== JSON.stringify(prevState.paragraphsFeedback)) {
            const {getBlocks} = select('core/editor');
            this.spellingIssueFeedback = highlightSpellingGrammar(paragraphsFeedback, getBlocks());
            this.complianceListFeedback = highlightComplianceList(paragraphsFeedback, getBlocks());
        }


        // Button clicked
        if(!optimizeButton.isBusy && prevState.optimizeButton.isBusy) {
            if(optimizeFeedback !== null && Object.keys(optimizeFeedback).length > 0) {
                this.setState({
                    revertButton: {
                        disabled: false,
                        visible: true,
                    }
                });
            }
        }

        if(optimizing && analyzing){
            this.isPublishReadyNotice(this.state.taskListFeedback);
            this.setState({
                optimizing: false,
                analyzing: false
            });
        }

    }

    callParagraphFeedback() {

        const requestArray = [];

        const {selectedProfile} = this.state;
        const {taskListFeedback} = this.props;
        const {getBlocks} = select('core/editor');
        for(const block of getBlocks()) {
            if(!ATOMICREACH_VALID_BLOCK_NAMES_ARRAY.includes(block.name))
                continue;

            const paragraphArray = {};

            paragraphArray.contentTypeName = "Article";
            paragraphArray.environment = "PRODUCTION";
            paragraphArray.measureNamesArray = ["rtSpellingGrammar", "rtComplianceList"];
            paragraphArray.content = encodeURIComponent(getHTMLFromBlock(block));
            paragraphArray.blockId = block.clientId;
            paragraphArray.blockName = block.name;

            if(selectedProfile) {
                paragraphArray['profileId'] = selectedProfile;
            }

            if(Object.keys(taskListFeedback).length) {
                const paragraphLength = taskListFeedback.data.PercentageParagraphsIdealWordCount;
                if(!paragraphLength.state) {
                    paragraphArray.readabilityTargetBucket = taskListFeedback.data.Readability.target;
                    paragraphArray.measureNamesArray.push("rtParagraphLength")

                }
            }

            paragraphArray.measureNamesArray = JSON.stringify(paragraphArray.measureNamesArray);
            requestArray.push(paragraphArray);
        }

        const url = ATOMICREACH_API_NAMESPACE + ATOMICREACH_API_PARAGRAPH_ANALYZE_ARTICLES;

        this.setAtomicReachApiCallInProgress(true);
        apiFetch({
            path: url,
            method: 'POST',
            data: {
                requestArray: JSON.stringify(requestArray)
            }
        }).then((feedback) => {
            this.setAtomicReachApiCallInProgress(false);
            this.setState({
                paragraphsFeedback: feedback,
            })

        }).catch((error) => {
            console.log(error);
        })


    }

    closePopover() {
        const {popoverState} = this.state;
        this.setState({
            popoverState: {
                ...popoverState,
                showPopover: false,
            },
        });
    };

    subscribeListener() {

        if(!this.componentIsMounted) return;


        const {isSavingPost, isAutosavingPost, isPublishingPost} = select('core/editor');
        const {getActiveGeneralSidebarName} = select('core/edit-post');
        const {isTyping} = select('core/editor');


        // Ensure Atomic Reach sidebar is active and user has clicked on the optimize button atleast once.
        if(getActiveGeneralSidebarName() === ATOMICREACH_SIDEBAR_NAME && this.state.optimizeButton.hasClicked) {
            if(this.hasEditorContentUpdated()) {
                this.setState({
                    hasChangedContent: true,
                });
            }
        }

        if(getActiveGeneralSidebarName() === ATOMICREACH_SIDEBAR_NAME &&
            isTyping() &&
            this.state.optimizeButton.disabled === true &&
            this.state.optimizeButton.isBusy === false) {
            this.reEnableOptimizeButton()
        }


        // When post is saving remove the highlighting.
        if((isSavingPost() || isPublishingPost()) && !isAutosavingPost()) {
            this.removeARHighlighting();
        }

    }

    getContent() {
        const {getBlocks} = select('core/editor');
        let content = '';
        for(const block of getBlocks()) {
            content += getHTMLFromBlock(block);
        }
        return content;
    }

    updateTitleContentState() {

        // set hasChangedContent to false every time update state with up to date content.

        this.setState({
            title: select('core/editor').getEditedPostAttribute('title'),
            content: removeARMarkerTags(select('core/editor').getEditedPostContent()),
            hasChangedContent: false,
        });

    }

    updateProfileState() {
        this.setState({
            profiles: this.props.profiles,
        });
    }

    updateSelectedProfile(value) {

        const {profiles} = this.props;
        let profileDetails = {};

        profiles.forEach((profile) => {
            if(value.profile == profile.value) {
                profileDetails = profile
            }
        });


        dispatch(ATOMICREACH_STORE).setProfile({
            profile: value
        });


        this.setState({
            selectedProfile: value.profile,
            selectedProfileDetails: profileDetails,
            tooltipVisibility: false,
            optimizeButton: {
                ...this.state.optimizeButton,
                disabled: isNaN(value.profile)
            }
        });
    }

    ProfilesDropdownList() {
        const {selectedProfile} = this.state;
        const {profiles} = this.props;
        return (
            <SelectControl
                // label={"Select Profile"}
                value={selectedProfile}
                options={profiles}
                onChange={
                    (profile) => {
                        if(profile !== 'Select a profile')
                            this.updateSelectedProfile({profile})
                    }
                }
            />
        )
    }

    OptimizeButtonClickHandler() {

        this.removeNotice('publishReadyNotice');
        this.totalOptimizeWordsToHighlight = 0;

        // disable and show spinner in the button
        this.setState({
            optimizeButton: {
                disabled: true,
                isBusy: true,
                hasClicked: true,
            },
            revertButton: {
                visible: false,
                disabled: false,
            },
            tooltipMessage: 'Your language is optimized',
            tooltipVisibility: true,
            optimizing:false,
            analyzing:false,
        });

        this.removeARHighlighting();

        this.callOptimizeApi();
        /*this.callTitleDocumentForOriginalContentApi().then(()=>{
        });*/


    }

    callOptimizeApi() {

        const {selectedProfile} = this.state;

        if(selectedProfile === null)
            return;

        this.setAtomicReachApiCallInProgress(true);
        const url = ATOMICREACH_API_NAMESPACE + ATOMICREACH_API_OPTIMIZE_ARTICLES;

        this.showNotice = true;

        apiFetch({
            path: url,
            method: 'POST',
            data: {
                profileId: selectedProfile,
                content: this.getContent(),
            }
        }).then((feedback) => {

            this.totalOptimizeWordsToHighlight = 0; // resets
            for(const ind in feedback) {
                this.totalOptimizeWordsToHighlight += Object.keys(feedback[ind]).length;
            }

            console.log("Debug: ", this.totalOptimizeWordsToHighlight, "words to highlight");




            dispatch(ATOMICREACH_STORE).setOptimizeFeedback(feedback);

            this.setAtomicReachApiCallInProgress(false);

            this.setState({
                optimizeFeedback: feedback,
                optimizing:true,
                optimizeButton: {
                    ...this.state.optimizeButton,
                    isBusy: false,
                }
            });

            this.optimizeFeedbackForRehighlighting = cloneDeep(feedback);

            this.processOptimizeFeedback();





        }).catch((err) => {
            console.error(err);
            this.reEnableOptimizeButton();
            this.setAtomicReachApiCallInProgress(false);
            this.totalOptimizeWordsToHighlight = 0;
            this.callTitleDocumentApi();
            this.showNotice = false;
        });
    }

    /*
    * Until https://developer.wordpress.org/block-editor/developers/block-api/block-annotations/ apis are available
    * we have a custom method of doing highlighting.
    * */
    processOptimizeFeedback() {
        const {optimizeFeedback} = this.state;
        const {getBlocks} = select('core/editor');

        if(this.totalOptimizeWordsToHighlight === 0) {
            console.log("no word replacement to do..exiting", optimizeFeedback);
            this.totalOptimizeWordsToHighlight = 0;
            this.callTitleDocumentApi();
            return;
        }

        /*New Optimize Feedback*/
        this.highlightOptimizeWords();

        debugCountHighlightedWords(getBlocks());

    }

    highlightOptimizeWords(isReHighlighting = false) {

        const {optimizeFeedback} = this.state;
        const {getBlocks} = select('core/editor');
        let blockIndex = 0;
        let optimizeFeedbackDuplicate = {};

        if(Object.entries(this.optimizeFeedbackForRehighlighting).length === 0 )
            return true;


        if(isReHighlighting) {
            optimizeFeedbackDuplicate = cloneDeep(this.optimizeFeedbackForRehighlighting);
        } else {
            optimizeFeedbackDuplicate = cloneDeep(optimizeFeedback);
        }

        // Only show the real number of highlighted words
        // @todo we should really only be using the number from here instead of the both setting the value in the logic above
        this.totalOptimizeWordsToHighlight = 0;
        for(const block of getBlocks()) {
            // Skip the paragraph which is not in ATOMICREACH_VALID_BLOCK_NAMES_ARRAY
            if(ATOMICREACH_VALID_BLOCK_NAMES_ARRAY.includes(block.name)) {

                if(optimizeFeedbackDuplicate[blockIndex].length === 0) {
                    // console.log("Skipping Paragraph ", blockIndex);
                    blockIndex++;
                    continue;
                }
                // Get HTML of block
                const blockHTML = getHTMLFromBlock(block, true);

                // Map the array for Mark Js
                Object.keys(optimizeFeedbackDuplicate[blockIndex]).map(function(key, val) {

                    optimizeFeedbackDuplicate[blockIndex][key]['start'] = optimizeFeedbackDuplicate[blockIndex][key]['idx'];
                    optimizeFeedbackDuplicate[blockIndex][key]['length'] = optimizeFeedbackDuplicate[blockIndex][key]['text'].length;
                    optimizeFeedbackDuplicate[blockIndex][key]['key'] = key;

                });

                // Create a node element for Mark Js to work with.
                const node = document.createElement('div');
                node.innerHTML = blockHTML.trim();

                // Initiate Mark Js
                const instance = new Mark(node);
                instance.markRanges(Object.values(optimizeFeedbackDuplicate[blockIndex]), {
                    "element": MARK_TAG_NAME,
                    "className": OPTIMIZED_WORD_CLASS,
                    "filter": ((textNode, range, term, counter) => {
                        // console.log("filter: ", textNode, range, term, counter);

                        if(term !== range.text) {
                            console.log("Unable to Highlight [" + range.text + "] | ", range, " | ", term);
                            return false;
                        }

                        if(!isReHighlighting)
                            this.adjustReHighlightingFeedback(blockIndex, range.key);

                        return true;
                    }), "each": ((node, range) => {
                        // node is the marked DOM element
                        // range is the corresponding range
                        this.totalOptimizeWordsToHighlight++;
                        node.setAttribute("id", "optimize_id_" + blockIndex + "_" + range.key);

                        if(!isReHighlighting)
                            node.innerText = range.replacement;

                        node.dataset.optimizeId = range.key;
                        node.dataset.blockIndex = blockIndex;
                        //@todo: add instance.

                    }),
                });

                updateBlockContent(block, node.innerHTML);

                blockIndex++;
            }
        }

    }

    adjustReHighlightingFeedback(blockIndex, tokenKey, replacement) {

        const text = this.optimizeFeedbackForRehighlighting[blockIndex][tokenKey]['text'];
        replacement = replacement || this.optimizeFeedbackForRehighlighting[blockIndex][tokenKey]['replacement'];
        this.optimizeFeedbackForRehighlighting[blockIndex][tokenKey]['text'] = replacement;
        this.optimizeFeedbackForRehighlighting[blockIndex][tokenKey]['replacement'] = text;
        if(replacement.length !== text.length) {
            const reHighlightingObjKeys = Object.keys(this.optimizeFeedbackForRehighlighting[blockIndex]);

            const startingIndex = reHighlightingObjKeys.indexOf(tokenKey);

            for(let i = (parseInt(startingIndex) + 1); i < reHighlightingObjKeys.length; i++) {
                const adjustment = (replacement.length - text.length);
                this.optimizeFeedbackForRehighlighting[blockIndex][reHighlightingObjKeys[i]]['idx'] += adjustment;
            }
        }

    }

    takeSnapShop() {
        const {getBlocks} = select('core/editor');
        let paragraphSnapShot = {};


        for(const block of getBlocks()) {
            let thisTextContent = removeARMarkerTags(getHTMLFromBlock(block, true)).trim();

            paragraphSnapShot[block.clientId] = thisTextContent;

        }

        dispatch(ATOMICREACH_STORE).setParagraphSnapShots(paragraphSnapShot);

    }

    checkForChanges() {
        const {getBlocks} = select('core/editor');
        const {paragraphSnapShot} = this.props;
        let hasChanges = false;

        if(Object.keys(paragraphSnapShot).length > 0) {

            for(const block of getBlocks()) {
                let thisTextContent = removeARMarkerTags(getHTMLFromBlock(block, true)).trim();

                if(paragraphSnapShot[block.clientId] !== thisTextContent) {
                    hasChanges = true;
                }

            }
        }


        return hasChanges;

    }

    reHighlightOptimizedWords() {

        this.highlightOptimizeWords(true);

    }

    /* Revert back to the original content.*/
    revertHighlightedWords() {
        const {getBlocks} = select('core/editor');
        const {optimizeFeedback} = this.state;

        for(const block of getBlocks()) {

            const blockHTML = getHTMLFromBlock(block, true);
            const DOM_nodes = jQuery('<div></div>').html(blockHTML);
            const optimizedWords = DOM_nodes.find('.optimizedWord');

            if(optimizedWords.length < 1)
                continue;

            for(const word of optimizedWords) {
                word.textContent = optimizeFeedback[word.dataset.blockIndex][word.dataset.optimizeId].text;
                word.outerHTML = word.innerHTML.trim();
            }

            updateBlockContent(block, DOM_nodes.html());

        }

        dispatch(ATOMICREACH_STORE).setOptimizeFeedback([]);
        dispatch(ATOMICREACH_STORE).setOptimizeParagraphs({optimizedParagraphs: {}});

        this.reEnableOptimizeButton();

    }

    domHasOptimizedWords(){
        const editor = document.querySelector('.edit-post-layout__content');
        let optimizedWords = jQuery(editor).find('.optimizedWord');

        return optimizedWords.length;
    }

    callTitleDocumentApi() {

        // Cancel if already a call in progress
        if(this.getAtomicReachApiCallInProgress())
            return;

        const {selectedProfile} = this.state;

        const title = select('core/editor').getEditedPostAttribute('title');

        this.setAtomicReachApiCallInProgress(true);

        const url = ATOMICREACH_API_NAMESPACE + ATOMICREACH_API_ANALYZE_ARTICLES;

        return apiFetch({
            path: url,
            method: 'POST',
            data: {
                profileId: selectedProfile,
                title,
                contentHtml: this.getContent(),
            }
        }).then((feedback) =>
            this.titleDocumentFeedback(feedback)).catch((err) =>
            (this.titleDocumentErrorFeedback(err)
            ));
    }

    titleDocumentFeedback(feedback) {

        dispatch(ATOMICREACH_STORE).setTaskListFeedback({
            taskListFeedback: feedback
        });
        // Do not remove the below log until PB approves.
        // PB needs the following log for his test.
        console.log('Arousal\tTargetBucket:[', feedback.data.Arousal.targetBucket, ']\tcurrentBucket:[', feedback.data.Arousal.bucket, "]\taverageArousalValue:[" + feedback.data.Arousal.debug.averageArousalValue + "}\t|\tReadability\tTargetBucket:[", feedback.data.Readability.target, ']\tcurrentBucket:[', feedback.data.Readability.actual, "]\treadabilityScore:[", feedback.data.Readability.readabilityScore, "]");

        this.callParagraphFeedback();
        this.setAtomicReachApiCallInProgress(false);

        this.setState({
            taskListFeedback: feedback,
            analyzing:true,
            optimizeButton: {
                ...this.state.optimizeButton,
            }
        });

    }

    titleDocumentErrorFeedback(err) {
        console.error(err);
        this.reEnableOptimizeButton();
        this.setAtomicReachApiCallInProgress(false);
    }

    getEngagementStr() {
        const {selectedProfileDetails} = this.state;

        let titleEngagement = this.engagementNames[selectedProfileDetails.titleEngagement];
        let bodyEngagement = this.engagementNames[selectedProfileDetails.bodyEngagement];
        let engagementStr = '';

        if(titleEngagement === bodyEngagement)
            engagementStr = titleEngagement;
        else
            engagementStr = `${titleEngagement} and ${bodyEngagement}`;

        return engagementStr;
    }

    checkForOptimization(feedback){

        let optimized =  true;
        if(typeof feedback !== 'undefined') {
            this.optimziedMeasures.forEach((measure) => {
                if (feedback.data[measure].state === false) {
                    optimized = false
                }
            });
        }else{
            optimized = false;
        }

        return optimized
    }

    isPublishReadyNotice(feedback) {
        if(!this.showNotice) {
            return;
        }

        if(this.getAtomicReachApiCallInProgress()) {
            console.trace();
            return;
        }

        const noticeId = "publishReadyNotice";
        let engagementStr = this.getEngagementStr();

        this.removeNotice(noticeId);


        if(this.checkForOptimization(feedback) && this.totalOptimizeWordsToHighlight === 0) {
            dispatch("core/notices")
                .createInfoNotice('Your language is already optimized', {
                    id: noticeId
                });

        } else if(this.totalOptimizeWordsToHighlight > 0) {
            const tense = (this.totalOptimizeWordsToHighlight !== 1) ? 'changes were' : 'change was';
            dispatch("core/notices")
                .createSuccessNotice(`Your language is now optimized for ${engagementStr}. ${this.totalOptimizeWordsToHighlight} ${tense} made.`, {
                    id: noticeId
                })
        }else if(this.totalOptimizeWordsToHighlight === 0 && this.checkForOptimization() === false) {
            this.setState({
                tooltipMessage: 'Your language cannot be optimized',
                tooltipVisibility: true
            });

            dispatch("core/notices")
                .createErrorNotice('Your article could not be optimized for this profile', {
                    id: noticeId
                });
        }

        this.showNotice = false;
    }


    removeNotice(noticeId) {

        const notices = select("core/notices").getNotices();

        if(notices.length > 0) {
            for(const notice of notices) {
                if(notice.id === noticeId) {
                    dispatch("core/notices").removeNotice(noticeId);
                }
            }
        }
    }

    disableOptimizeButton() {
        this.setState({
            optimizeButton: {
                ...this.state.optimizeButton,
                disabled: true,
                isBusy: false,
            }
        })
    }

    reEnableOptimizeButton() {

        const {selectedProfile} = this.props;

        if(Object.keys(selectedProfile).length > 0) {
            this.setState({
                optimizeButton: {
                    ...this.state.optimizeButton,
                    disabled: false,
                    isBusy: false,
                },
                tooltipVisibility: false
            })
        }
    }


    cardSelectionCallback(measure, isHighlightOn) {
        const {paragraphsFeedback} = this.state;
        const {getBlocks} = select('core/editor');

        if(isHighlightOn) {
            highlightParagraphLength(paragraphsFeedback);
        } else {
            this.removeARHighlighting();
            this.reHighlightOptimizedWords();
            highlightSpellingGrammar(paragraphsFeedback, getBlocks());
            highlightComplianceList(paragraphsFeedback, getBlocks());

        }
    }

    getAtomicReachApiCallInProgress() {
        return this.atomicReachApiCallInProgress;
    }

    setAtomicReachApiCallInProgress(state) {
        this.atomicReachApiCallInProgress = state;
    }

    hasEditorContentUpdated() {
        const {title, content} = this.state;
        const {getEditedPostContent, getEditedPostAttribute} = select('core/editor');

        return (title !== getEditedPostAttribute('title') || content !== removeARMarkerTags(getEditedPostContent()))
    }

    removeARHighlighting() {
        if(this.clearingHiglights) {
            // console.log("cancel clearing highlighting because it is already running.");
            return;
        }
        this.clearingHiglights = true;
        const {getBlocks} = select('core/editor');
        for(const block of getBlocks()) {
            cleanARHighlightingFromBlock(block);
        }
        this.clearingHiglights = false;
    }

    removeTempARHighlighting() {
        this.hideARHighlighting()
    }


    hideARHighlighting() {
        document.querySelector(MARK_TAG_NAME).classList.add('hideHighlighting');
    }

    wordReplacedFromPopover() {
        this.setState({
            showPopover: false,
        });
    };

    render() {
        const {
            taskListFeedback,
            optimizeButton,
            popoverState,
            revertButton,
            tooltipMessage,
            tooltipVisibility
        } = this.state;
        const {
            optimizeFeedback,
            optimizedParagraphs
        } = this.props;
        const editor = document.querySelector('.edit-post-layout__content');
        const {
            popoverFeedback,
            popoverOriginalWord,
            popoverId,
            showPopover,
            type,
            showAddToDictionary,
        } = popoverState;

        // console.log('-------------t a s k l i s t -------------');
        // console.log(taskListFeedback);
        // console.log(optimizeButton);

        return (
            <div id={'atomicreach-sidebar-wrapper'}>
                <div className={'profile-button-wrapper'}>
                    <div id={'atomicreach-profile-wrapper'}>{this.ProfilesDropdownList()} </div>
                    <div id={'atomicreach-optimize-button-wrapper'}>
                        <TooltipContainer
                            optimizeButton={optimizeButton}
                            clickHandler={(e) => {
                                this.OptimizeButtonClickHandler(e)
                            }
                            }
                            text={tooltipMessage}
                            position={'top'}
                            visibility={tooltipVisibility}
                        />
                        {(revertButton.visible) ?
                            <Button
                                id={'atomicreach-revert-button'}
                                disabled={revertButton.disabled}
                                onClick={(e) => {
                                    e.preventDefault();
                                    this.setState({
                                        revertButton: {
                                            disabled: true,
                                            visible: true,
                                        }
                                    });
                                    this.revertHighlightedWords();
                                }
                                }
                                isLink

                            >Revert Optimized Words
                            </Button>
                            : null}
                    </div>
                </div>

                <hr className={`ar-line ${(Object.keys(taskListFeedback).length === 0 || optimizeButton.isBusy) ? '' : 'hide'}`}/>
                <SidebarBodyFeedback
                    panelTitle={'Improvements'}
                    feedback={taskListFeedback}
                    optimizeStatus={optimizeButton}
                    measures={this.bodyMeasures}
                    cardSelectionCallback={this.cardSelectionCallback}
                />

                <SidebarTitleFeedback
                    titleMeasures={this.titleMeasures}
                    bodyMeasures={this.bodyMeasures}
                    feedback={taskListFeedback}
                    optimizeStatus={optimizeButton}
                />

                {ReactDOM.createPortal(
                    <GenericPopover
                        isOpen={showPopover}
                        type={type}
                        originalText={popoverOriginalWord}
                        originalClickable={optimizeButton.hasClicked}
                        popoverFeedback={popoverFeedback}
                        wordReplacedCallback={(e) => {

                            let target = document.getElementById(popoverId); // ensure that the document never losses the active element
                            replaceActiveWord(e.target, target);

                            if(target.classList.contains(OPTIMIZED_WORD_CLASS)) {
                                const optimizedId = target.getAttribute('data-optimize-id');
                                const blockIndex = target.dataset.blockIndex;
                                this.adjustReHighlightingFeedback(blockIndex, optimizedId, target.innerHTML);
                            }


                            let blockParagraph = target.closest('p');
                            const block = select("core/editor").getSelectedBlock();

                            updateBlockContent(block, blockParagraph.innerHTML);
                            this.closePopover();
                        }}
                        close={() => {
                            this.closePopover();
                        }}
                        repositionPopover={doRePositioningXYEditor}
                        target={() => popoverId}
                        wordFlags
                        flagFunction={(replaceWord, feedbackType) => {
                            let target = document.getElementById(popoverId); // ensure that the document never losses the active element
                            let paragraphElement = target.closest('p');
                            flagWordEditor(replaceWord, target, feedbackType, paragraphElement, popoverOriginalWord);
                        }}
                        className="editor-popover"
                        message={''}
                        addToDictionary={showAddToDictionary && (() => {
                            this.closePopover();
                            let target = document.getElementById(popoverId);
                            addToDictionary(target).then(() => {
                                target.outerHTML = target.innerHTML
                            });


                        })}
                    />, editor)
                }
            </div>
        )
    }


}

export default SidebarWrapper;
